@@ -38,6 +38,7 @@
 #include "document.h"
 #include "view.h"
 #include "application.h"
+#include "tool.h"
 #include <math.h>
 #include <unistd.h>
 
@@ -46,13 +47,15 @@ static void do_export_to_ghemical (gcpMolecule* pMol)
 	pMol->ExportToGhemical ();
 }
 
-static void do_select_alignment (gcpMolecule* pMol)
+static void do_select_alignment (GObject *action, gcpMolecule* pMol)
 {
-	pMol->SelectAlignmentItem ();
+	Object *object = (Object*) g_object_get_data (action, "item");
+	pMol->SelectAlignmentItem (object);
 }
 
 gcpMolecule::gcpMolecule(TypeId Type): Object(Type)
 {
+	m_Alignment = NULL;
 }
 
 gcpMolecule::gcpMolecule(gcpAtom* pAtom): Object(MoleculeType)
@@ -95,10 +98,13 @@ void gcpMolecule::AddBond(gcpBond* pBond)
 {
 	m_Bonds.push_back(pBond);
 	AddChild(pBond);
+	EmitSignal (OnChangedSignal);
 }
 
 void gcpMolecule::Remove(Object* pObject)
 {
+	if (pObject == m_Alignment)
+		m_Alignment = NULL;
 	switch (pObject->GetType())
 	{
 		case AtomType:
@@ -302,79 +308,86 @@ bool gcpMolecule::Merge(gcpMolecule* pMolecule, bool RemoveDuplicates)
 		m_Cycles.push_back(pCycle);
 		pMolecule->m_Cycles.pop_front();
 	}
+	Object* pObj = pMolecule->GetParent ();
 	delete pMolecule;
+	pObj->EmitSignal (OnChangedSignal);
 	if (RemoveDuplicates)
 		UpdateCycles ();
+	EmitSignal (OnChangedSignal);
 	return true;
 }
 
-bool gcpMolecule::Load(xmlNodePtr node)
+bool gcpMolecule::Load (xmlNodePtr node)
 {
-	char* tmp;
+	char* buf;
 	xmlNodePtr child;
 	Object* pObject;
-	gcpDocument* pDoc = (gcpDocument*)GetDocument();
+	gcpDocument* pDoc = (gcpDocument*) GetDocument ();
 
-	tmp = (char*) xmlGetProp (node, (xmlChar*) "id");
-	if (tmp) {
-		SetId (tmp);
-		xmlFree (tmp);
+	buf = (char*) xmlGetProp (node, (xmlChar*) "id");
+	if (buf) {
+		SetId (buf);
+		xmlFree (buf);
 	}
-	child = GetNodeByName(node, "atom");
-	while (child)
-	{
-		pObject = new gcpAtom();
+	child = GetNodeByName (node, "atom");
+	while (child) {
+		pObject = new gcpAtom ();
 		if (pDoc)
 			AddChild (pObject);
-		if (!pObject->Load(child)) 
-		{
+		if (!pObject->Load (child)) {
 			delete pObject;
 			return false;
 		}
-		if (pDoc) pDoc->AddAtom((gcpAtom*)pObject);
-		AddAtom((gcpAtom*)pObject);
-		child = GetNextNodeByName(child->next, "atom");
+		if (pDoc)
+			pDoc->AddAtom ((gcpAtom*) pObject);
+		AddAtom ((gcpAtom*) pObject);
+		child = GetNextNodeByName (child->next, "atom");
 	}
 	
-	child = GetNodeByName(node, "fragment");
-	while (child)
-	{
-		pObject = new gcpFragment();
-		AddChild(pObject);
-		if (!pObject->Load(child)) 
-		{
+	child = GetNodeByName (node, "fragment");
+	while (child) {
+		pObject = new gcpFragment ();
+		AddChild (pObject);
+		if (!pObject->Load (child))  {
 			delete pObject;
 			return false;
 		}
-		if (pDoc) pDoc->AddFragment((gcpFragment*)pObject);
-		AddFragment((gcpFragment*)pObject);
-		((gcpFragment*)pObject)->AnalContent();
-		child = GetNextNodeByName(child->next, "fragment");
+		if (pDoc)
+			pDoc->AddFragment ((gcpFragment*) pObject);
+		AddFragment ((gcpFragment*) pObject);
+		((gcpFragment*) pObject)->AnalContent ();
+		child = GetNextNodeByName (child->next, "fragment");
 	}
 
-	child = GetNodeByName(node, "bond");
-	while (child)
-	{
-		pObject = new gcpBond();
-		AddBond((gcpBond*)pObject);
-		if (!pObject->Load(child))
-		{
+	child = GetNodeByName (node, "bond");
+	while (child) {
+		pObject = new gcpBond ();
+		AddBond ((gcpBond*) pObject);
+		if (!pObject->Load (child)) {
 			delete pObject;
-			m_Bonds.remove((gcpBond*)pObject);
+			m_Bonds.remove ((gcpBond*) pObject);
 			return false;
 		}
-		if (pDoc) pDoc->AddBond((gcpBond*)pObject);
-		child = GetNextNodeByName(child->next, "bond");
+		if (pDoc)
+			pDoc->AddBond ((gcpBond*) pObject);
+		child = GetNextNodeByName (child->next, "bond");
 	}
-	if (!m_Atoms.empty())
-	{
-		gcpAtom* pAtom = m_Atoms.front();
-		std::list<gcpAtom*>::iterator i = m_Atoms.begin();
+	if (!m_Atoms.empty ()) {
+		gcpAtom* pAtom = m_Atoms.front ();
+		std::list<gcpAtom*>::iterator i = m_Atoms.begin ();
 		i++;
-		for (; i != m_Atoms.end(); i++) (*i)->SetParent(NULL);
-		gcpChain* pChain = new gcpChain(this, pAtom); //will find the cycles
+		for (; i != m_Atoms.end (); i++)
+			(*i)->SetParent (NULL);
+		gcpChain* pChain = new gcpChain (this, pAtom); //will find the cycles
 		delete pChain;
 	}
+	buf = (char*) xmlGetProp (node, (const xmlChar*) "valign");
+	if (buf) {
+		m_Alignment = GetDescendant (buf);
+		xmlFree (buf);
+		if (!m_Alignment)
+			return false;
+	}
 	return true;
 }
 
@@ -437,6 +450,8 @@ Object* gcpMolecule::GetAtomAt(double x, double y, double z)
 	
 double gcpMolecule::GetYAlign ()
 {
+	if (m_Alignment)
+		return m_Alignment->GetYAlign ();
 	double y, maxy = - DBL_MAX, miny = DBL_MAX;
 	std::list<gcpAtom*>::iterator i = m_Atoms.begin(), end = m_Atoms.end();
 	for (; i != end; i++) {
@@ -457,7 +472,7 @@ double gcpMolecule::GetYAlign ()
 	return (miny + maxy) / 2.0;
 }
 
-bool gcpMolecule::BuildContextualMenu (GtkUIManager *UIManager)
+bool gcpMolecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object)
 {
 	bool result = false;
 	GtkActionGroup *group = gtk_action_group_new ("molecule");
@@ -469,18 +484,19 @@ bool gcpMolecule::BuildContextualMenu (GtkUIManager *UIManager)
 		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menuitem action='ghemical'/></popup></ui>", -1, NULL);
 		result = true;
 	}
-/*	if (m_Bonds.size ()) {
+	if (m_Bonds.size ()) {
 		action = gtk_action_new ("select-align", _("Select alignment item"), NULL, NULL);
-		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_select_alignment), this);
+		g_signal_connect (action, "activate", G_CALLBACK (do_select_alignment), this);
+		g_object_set_data (G_OBJECT (action), "item", object);
 		gtk_action_group_add_action (group, action);
 		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menuitem action='select-align'/></popup></ui>", -1, NULL);
 		result = true;
-	}*/
+	}
 	if (result)
 		gtk_ui_manager_insert_action_group (UIManager, group, 0);
 	else
 		g_object_unref (group);
-	return result | GetParent ()->BuildContextualMenu (UIManager);
+	return result | GetParent ()->BuildContextualMenu (UIManager, object);
 }
 
 void gcpMolecule::ExportToGhemical ()
@@ -561,6 +577,18 @@ void gcpMolecule::ExportToGhemical ()
 	g_free (command_line);
 }
 
-void gcpMolecule::SelectAlignmentItem ()
+void gcpMolecule::SelectAlignmentItem (Object *child)
 {
+	m_Alignment = (m_Alignment != child)? child: NULL;
+	EmitSignal (OnChangedSignal);
+}
+
+xmlNodePtr gcpMolecule::Save (xmlDocPtr xml)
+{
+	xmlNodePtr node = Object::Save (xml);
+	if (!node)
+		return NULL;
+	if (m_Alignment)
+		xmlNewProp (node, (const xmlChar*) "valign", (const xmlChar*) m_Alignment->GetId ());
+	return node;
 }
