@@ -4,7 +4,7 @@
  * GChemPaint library
  * molecule.h 
  *
- * Copyright (C) 2001-2004
+ * Copyright (C) 2001-2005
  *
  * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
@@ -54,6 +54,8 @@ public:
 	virtual bool BuildContextualMenu (GtkUIManager *UIManager, Object *object);
 	void ExportToGhemical ();
 	void SelectAlignmentItem (Object *child);
+	void Lock (bool LockState) {m_IsLoading = LockState;}
+	string GetAlignmentId () {return (m_Alignment)? m_Alignment->GetId (): "";}
 
 private:
 	list<gcpCycle*> m_Cycles;
