@@ -4,7 +4,7 @@
  * GChemPaint library
  * reaction-step.cc 
  *
- * Copyright (C) 2004
+ * Copyright (C) 2004-2005
  *
  * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
@@ -28,6 +28,7 @@
 #include "reaction-step.h"
 #include "reaction.h"
 #include "reactant.h"
+#include "reaction-arrow.h"
 #include "reaction-operator.h"
 #include "document.h"
 #include "view.h"
@@ -84,6 +85,40 @@ gcpReactionStep::gcpReactionStep (gcpReaction *reaction, map<double, Object*>& C
 
 gcpReactionStep::~gcpReactionStep ()
 {
+	set<gcpReactionArrow *>::iterator i, end = m_Arrows.end();
+	for (i = m_Arrows.begin (); i != end; i++)
+		(*i)->RemoveStep (this);
+	/* If there are no chidren, don't care and exit */
+	if (!GetChildrenNumber ())
+		return;
+	/* Now, destroy children and add the reactant contents to the reaction parent,
+	not to the reaction */
+	gcpDocument *pDoc = reinterpret_cast<gcpDocument *> (GetDocument ());
+	gcpOperation *pOp = pDoc->GetCurrentOperation ();
+	gcpReaction *Reaction = reinterpret_cast<gcpReaction *> (GetParent ());
+	if (!Reaction)
+		return;
+	Object *pObj = Reaction->GetParent (), *Child, *Group = Reaction->GetGroup ();
+	map<string, Object *>::iterator j;
+	gcpReactant *Reactant;
+	while (HasChildren ()) {
+		Child = GetFirstChild (j);
+		if (Child->GetType () == ReactionOperatorType) {
+			pDoc->Remove (Child);
+			continue;
+		}
+		Reactant = reinterpret_cast<gcpReactant *> (Child);
+		Child = Reactant->GetStoechChild ();
+		if (Child)
+			pDoc->Remove (Child);
+		Child = Reactant->GetChild ();
+		if (Child) {
+			Child->SetParent (pObj);
+			if (pOp && !Group)
+				pOp->AddObject (Child, 1);
+		}
+		delete Reactant;
+	}
 }
 	
 xmlNodePtr gcpReactionStep::Save (xmlDocPtr xml)
@@ -223,3 +258,11 @@ bool gcpReactionStep::OnSignal (SignalId Signal, Object *Child)
 	} else
 		return true;
 }
+
+void gcpReactionStep::RemoveArrow (gcpReactionArrow *arrow) {
+	m_Arrows.erase (arrow);
+	if (m_Arrows.empty ()) {
+		// if there is no more arrows this is no more a reaction step
+		delete this;
+	}
+}
