@@ -4,9 +4,9 @@
  * GChemPaint atoms plugin
  * chargetool.cc 
  *
- * Copyright (C) 2003-2004
+ * Copyright (C) 2003-2005
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@n>ormalesup.org
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -302,13 +302,14 @@ void gcpChargeTool::OnRelease()
 		char charge = pAtom->GetCharge();
 		gcpDocument* pDoc = m_pView->GetDoc();
 		gcpOperation* pOp = pDoc-> GetNewOperation(GCP_MODIFY_OPERATION);
-		m_pObject = m_pObject->GetMolecule ();
+		m_pObject = m_pObject->GetGroup ();
 		pOp->AddObject(m_pObject, 0);
 		pAtom->SetCharge(m_Charge);
 		pAtom->SetChargePosition (m_Pos, m_Pos == m_DefaultPos, m_dAngle,
 			(m_nState & GDK_SHIFT_MASK)? m_dDist / m_dZoomFactor: 0.);
 		pAtom->Update();
 		m_pView->Update(m_pObject);
+		pAtom->EmitSignal (OnChangedSignal);
 		pOp->AddObject(m_pObject, 1);
 		pDoc->FinishOperation();
 	}
