@@ -4,9 +4,9 @@
  * GChemPaint bonds plugin
  * bondtool.cc
  *
- * Copyright (C) 2001-2004
+ * Copyright (C) 2001-2005
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -35,37 +35,38 @@
 #include "lib/bond.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 
-gcpBondTool::gcpBondTool(gcpApplication *App, string ToolId, unsigned nPoints): gcpTool(App, ToolId)
+gcpBondTool::gcpBondTool (gcpApplication *App, string ToolId, unsigned nPoints): gcpTool (App, ToolId)
 {
 	points = (nPoints)? gnome_canvas_points_new (nPoints): NULL;
 	m_pOp = NULL;
 }
 
-gcpBondTool::~gcpBondTool()
+gcpBondTool::~gcpBondTool ()
 {
-	if (points) gnome_canvas_points_free(points);
+	if (points) gnome_canvas_points_free (points);
 }
 
-bool gcpBondTool::OnClicked()
+bool gcpBondTool::OnClicked ()
 {
 	double x1, y1, x2, y2;
-	int i = 1;
+	int i;
 	m_pAtom = NULL;
 	m_pItem = NULL;
 	m_bChanged = false;
+	m_dAngle = 0.;
 	gcpBond* pBond;
-	gcpDocument* pDoc = m_pView->GetDoc();
+	gcpDocument* pDoc = m_pView->GetDoc ();
 	if (m_pObject)
 	{
-		TypeId Id = m_pObject->GetType();
+		TypeId Id = m_pObject->GetType ();
 		switch (Id)
 		{
 		case BondType:
-			pBond = (gcpBond*)m_pObject;
-			m_pAtom = (gcpAtom*)pBond->GetAtom(0);
-			m_pAtom->GetCoords(&m_x0, &m_y0, NULL);
-			m_pAtom = (gcpAtom*)pBond->GetAtom(1);
-			m_pAtom->GetCoords(&m_x1, &m_y1, NULL);
+			pBond = (gcpBond*) m_pObject;
+			m_pAtom = (gcpAtom*) pBond->GetAtom (0);
+			m_pAtom->GetCoords (&m_x0, &m_y0, NULL);
+			m_pAtom = (gcpAtom*) pBond->GetAtom (1);
+			m_pAtom->GetCoords (&m_x1, &m_y1, NULL);
 			m_x0 *= m_dZoomFactor;
 			m_y0 *= m_dZoomFactor;
 			m_x1 *= m_dZoomFactor;
@@ -73,9 +74,9 @@ bool gcpBondTool::OnClicked()
 			points->coords[0] = m_x0;
 			points->coords[1] = m_y0;
 			m_bChanged = true;
-			m_pOp = pDoc-> GetNewOperation(GCP_MODIFY_OPERATION);
-			m_pOp->AddObject(m_pObject, 0);
-			UpdateBond();
+			m_pOp = pDoc->GetNewOperation (GCP_MODIFY_OPERATION);
+			m_pOp->AddObject (m_pObjectGroup, 0);
+			UpdateBond ();
 			return true;
 		case AtomType:
 			if (!((gcpAtom*)m_pObject)->AcceptNewBonds()) return false;
@@ -83,7 +84,33 @@ bool gcpBondTool::OnClicked()
 			m_x0 *= m_dZoomFactor;
 			m_y0 *=  m_dZoomFactor;
 			points->coords[0] = m_x0;
-			points->coords[1] = m_y1 = m_y0;
+			points->coords[1] = m_y0;
+			/* search  preferred orientation for new bond */
+			i = ((gcpAtom*) m_pObject)->GetBondsNumber ();
+			switch (i) {
+				case 0:
+					break;
+				case 1: {
+					break;
+				}
+				case 2: {
+					double a1, a2;
+					map<Atom*, Bond*>::iterator i;
+					gcpBond* bond = (gcpBond*) ((Atom*) m_pObject)->GetFirstBond (i);
+					a1 = bond->GetAngle2D ((gcpAtom*) m_pObject);
+					bond = (gcpBond*) ((Atom*) m_pObject)->GetNextBond (i);
+					a2 = bond->GetAngle2D ((gcpAtom*) m_pObject);
+					m_dAngle = (a1 + a2) / 2.;
+					a2 = fabs (a2 - m_dAngle);
+					if (a2 < 90.)
+						m_dAngle += 180.;
+					if (m_dAngle > 360.)
+						m_dAngle -= 360.;
+					break;
+				}
+				default:
+					break;
+			}
 			break;
 		default:
 			return false;
@@ -94,12 +121,15 @@ bool gcpBondTool::OnClicked()
 		points->coords[0] = m_x0;
 		points->coords[1] = m_y0;
 	}
+	double a = m_dAngle * M_PI / 180.;
+	m_x1 =  m_x0 + DefaultBondLength * m_dZoomFactor * cos (a);
+	m_y1 =  m_y0 - DefaultBondLength * m_dZoomFactor * sin (a);
 	GnomeCanvasItem* pItem = gnome_canvas_get_item_at(GNOME_CANVAS(m_pWidget), m_x1, m_y1);
 	if (pItem == (GnomeCanvasItem*)m_pBackground) pItem = NULL;
 	Object* pObject = NULL;
 	if (pItem) pObject = (Object*)g_object_get_data(G_OBJECT(pItem), "object");
 	m_pAtom = NULL;
-	if (pObject)
+	if (pObject && pObject != m_pObject)
 	{
 		if ((pObject->GetType() == BondType) || (pObject->GetType() == FragmentType))
 		{
@@ -120,11 +150,6 @@ bool gcpBondTool::OnClicked()
 		m_dAngle = atan(-m_y/m_x) * 90 / 1.570796326794897;
 		if (m_x < 0) m_dAngle += 180;
 	}
-	else
-	{
-		m_x1 =  m_x0 + DefaultBondLength * m_dZoomFactor;
-		m_dAngle = 0;
-	}
 	char tmp[32];
 	snprintf(tmp, sizeof(tmp) - 1, _("Orientation: %g"), m_dAngle);
 	m_pApp->SetStatusText(tmp);
@@ -192,7 +217,10 @@ void gcpBondTool::OnDrag()
 			m_x = m_x1 - m_x0;
 			m_y = m_y1 - m_y0;
 			dAngle = atan(-m_y/m_x) * 90 / 1.570796326794897;
-			if (m_x < 0) dAngle += 180;
+			if (isnan (dAngle))
+				dAngle = m_dAngle;
+			else if (m_x < 0.)
+				dAngle += 180.;
 		}
 		else
 		{
@@ -207,7 +235,10 @@ void gcpBondTool::OnDrag()
 			{
 				dAngle = atan(-m_y/m_x) * 180 / M_PI;
 				if (!(m_nState & GDK_CONTROL_MASK)) dAngle = rint(dAngle / 5) * 5;
-				if (m_x < 0) dAngle += 180;
+				if (isnan (dAngle))
+					dAngle = m_dAngle;
+				else if (m_x < 0.)
+					dAngle += 180.;
 			}
 			m_dAngle = dAngle;
 			if (m_nState & GDK_SHIFT_MASK)
@@ -247,23 +278,21 @@ void gcpBondTool::OnRelease()
 		m_pOp = NULL;
 		return;
 	}
-	if ((m_pObject) && (m_pObject->GetType() == BondType))
-	{
-		FinalizeBond();
-		gcpAtom* pAtom = (gcpAtom*) ((gcpBond*)m_pObject)->GetAtom(0);
+	if ((m_pObject) && (m_pObject->GetType() == BondType)) {
+		FinalizeBond ();
+		gcpAtom* pAtom = (gcpAtom*) ((gcpBond*) m_pObject)->GetAtom (0);
 		pAtom->Update ();
-		m_pView->Update(pAtom);
-		pAtom = (gcpAtom*) ((gcpBond*)m_pObject)->GetAtom(1);
+		m_pView->Update (pAtom);
+		pAtom = (gcpAtom*) ((gcpBond*) m_pObject)->GetAtom (1);
 		pAtom->Update ();
-		m_pView->Update(pAtom);
-		m_pOp->AddObject(m_pObject, 1);
-		pDoc->FinishOperation();
+		m_pView->Update (pAtom);
+		m_pOp->AddObject (m_pObjectGroup, 1);
+		pDoc->FinishOperation ();
 		m_pOp = NULL;
 		return;
-	}
-	else 
-	{
-		if (m_pOp) pDoc->AbortOperation();
+	} else {
+		if (m_pOp)
+			pDoc->AbortOperation();
 		m_pOp = NULL;
 	}
 	m_pApp->ClearStatus();
@@ -289,19 +318,27 @@ void gcpBondTool::OnRelease()
 	}
 	gcpAtom* pAtom;
 	gcpBond* pBond;
-	if (!m_pObject)
-	{
+	if (!m_pObject) {
 		//Add an atom at (x0, y0)
-		pAtom = new gcpAtom(m_pApp->GetCurZ(), m_x0 / m_dZoomFactor, m_y0 / m_dZoomFactor, 0);
-		pDoc->AddAtom(pAtom);
+		pAtom = new gcpAtom (m_pApp->GetCurZ(), m_x0 / m_dZoomFactor, m_y0 / m_dZoomFactor, 0);
+		pDoc->AddAtom (pAtom);
 		m_pObject = pAtom;
+	} else {
+		pObject = m_pObject->GetGroup ();
+		if (pObject)
+			ModifiedObjects.insert (pObject->GetId ());
 	}
 	if (m_pObject->GetType() == AtomType)
 	{
 		if (m_pAtom) {
 			if (m_pObject == m_pAtom) {
+				ModifiedObjects.clear ();
 				return;
 			}
+			pObject = m_pObject->GetGroup ();
+			if (!pObject)
+				throw runtime_error (_("Invalid document tree, please file a bug report"));
+			ModifiedObjects.insert (pObject->GetId ());
 			pAtom = m_pAtom;
 		} else {
 			pAtom = new gcpAtom(m_pApp->GetCurZ(), m_x1 / m_dZoomFactor, m_y1 / m_dZoomFactor, 0);
@@ -311,7 +348,7 @@ void gcpBondTool::OnRelease()
 		if (pBond)
 		{
 			m_pOp = pDoc-> GetNewOperation(GCP_MODIFY_OPERATION);
-			m_pOp->AddObject(pBond, 0);
+			m_pOp->AddObject(pBond->GetGroup (), 0);
 			if (pBond->GetType() == NormalBondType)	pBond->IncOrder();
 			m_pObject = pBond;
 			m_bChanged = true;
@@ -323,17 +360,37 @@ void gcpBondTool::OnRelease()
 			pAtom->Update ();
 			m_pView->Update(pAtom);
 			m_pView->Update(pBond);
-			m_pOp->AddObject(pBond, 1);
+			m_pOp->AddObject(pBond->GetGroup (), 1);
 			pDoc->FinishOperation();
 			m_pOp = NULL;
 		}
 		else
 		{
+			// Push modified objects in Operation
+			if (ModifiedObjects.size ()) {
+				m_pOp = pDoc-> GetNewOperation(GCP_MODIFY_OPERATION);
+				set<string>::iterator it, end = ModifiedObjects.end();
+				for (it = ModifiedObjects.begin(); it != end; it++)
+					m_pOp->AddObject (pDoc->GetDescendant ((*it).c_str ()), 0);
+			}
 			pBond = new gcpBond((gcpAtom*)m_pObject, pAtom, 1);
 			SetType(pBond);
 			pDoc->AddBond(pBond);
+			if (m_pOp) {
+				set<string>::iterator it, end = ModifiedObjects.end();
+				for (it = ModifiedObjects.begin(); it != end; it++) {
+					pObject = pDoc->GetDescendant ((*it).c_str ());
+					if (pObject)
+						m_pOp->AddObject (pObject, 1);
+				}
+			} else {
+				m_pOp = pDoc-> GetNewOperation(GCP_ADD_OPERATION);
+				m_pOp->AddObject (pBond->GetMolecule ());
+			}
+			pDoc->FinishOperation();
 		}
 	}
+	ModifiedObjects.clear ();
 }
 
 void gcpBondTool::Draw()
@@ -382,11 +439,14 @@ void gcpBondTool::FinalizeBond()
 	if (m_bChanged)
 	{
 		gcpBond* pBond = (gcpBond*)m_pObject;
-		if (pBond->GetType() != NormalBondType)
-		{
-			pBond->SetType(NormalBondType);
+		if (pBond->GetType () == NormalBondType) {
+			pBond->Revert ();
+			m_pView->Update (m_pObject);
+		} else {
+			pBond->SetType (NormalBondType);
+			m_pView->Remove (m_pObject);
+			m_pView->AddObject (m_pObject);
 		}
-		m_pView->Update(pBond);
 	}
 	else ((gcpBond*)m_pObject)->SetOrder(BondOrder);
 	m_pView->Update(((gcpBond*)m_pObject)->GetAtom(0));
@@ -448,9 +508,14 @@ void gcpUpBondTool::FinalizeBond()
 	if (m_bChanged)
 	{
 		gcpBond* pBond = (gcpBond*)m_pObject;
-		if (pBond->GetType() == UpBondType) pBond->Revert();
-		else pBond->SetType(UpBondType);
-		m_pView->Update(m_pObject);
+		if (pBond->GetType () == UpBondType) {
+			pBond->Revert ();
+			m_pView->Update (m_pObject);
+		} else {
+			pBond->SetType (UpBondType);
+			m_pView->Remove (m_pObject);
+			m_pView->AddObject (m_pObject);
+		}
 	}
 }
 
@@ -543,9 +608,14 @@ void gcpDownBondTool::FinalizeBond()
 	if (m_bChanged)
 	{
 		gcpBond* pBond = (gcpBond*)m_pObject;
-		if (pBond->GetType() == DownBondType) pBond->Revert();
-		else pBond->SetType(DownBondType);
-		m_pView->Update(m_pObject);
+		if (pBond->GetType () == DownBondType) {
+			pBond->Revert ();
+			m_pView->Update (m_pObject);
+		} else {
+			pBond->SetType (DownBondType);
+			m_pView->Remove (m_pObject);
+			m_pView->AddObject (m_pObject);
+		}
 	}
 }
 
@@ -609,10 +679,13 @@ void gcpSquiggleBondTool::FinalizeBond()
 	if (m_bChanged)
 	{
 		gcpBond* pBond = (gcpBond*)m_pObject;
-		if (pBond->GetType() != UndeterminedBondType)
-		{
-			pBond->SetType(UndeterminedBondType);
-			m_pView->Update(m_pObject);
+		if (pBond->GetType () == UndeterminedBondType) {
+			pBond->Revert ();
+			m_pView->Update (m_pObject);
+		} else {
+			pBond->SetType (UndeterminedBondType);
+			m_pView->Remove (m_pObject);
+			m_pView->AddObject (m_pObject);
 		}
 	}
 }
