@@ -6,7 +6,7 @@
  *
  * Copyright (C) 2002-2004
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -91,6 +91,7 @@ bool gcpTextTool::OnClicked()
 		g_object_set(G_OBJECT(m_Active), "editable", true, "cursor_visible", true, NULL);
 		m_CurNode = ((gcpText*)m_pObject)->SaveSelected();
 		m_InitNode = ((gcpText*)m_pObject)->SaveSelected();
+		m_pApp->ActivateMenu ("Image", false);
 	}
 	return true;
 }
@@ -247,12 +248,12 @@ bool gcpTextTool::Unselect()
 	}
 	xmlBufferPtr initbuf = xmlBufferCreate();
 	xmlBufferPtr endbuf = xmlBufferCreate();
-	xmlNodeDump(initbuf, m_pApp->GetXmlDoc(), m_InitNode, 0, 0);
-	xmlNodeDump(endbuf, m_pApp->GetXmlDoc(), m_CurNode, 0, 0);
+	xmlNodeDump (initbuf, m_pApp->GetXmlDoc(), m_InitNode, 0, 0);
+	xmlNodeDump (endbuf, m_pApp->GetXmlDoc(), m_CurNode, 0, 0);
 	if (strcmp((char*)initbuf->content,(char*)endbuf->content))
 	{
-		char* initval = (char*)xmlNodeGetContent(m_InitNode);
-		char* endval = (char*)xmlNodeGetContent(m_CurNode);
+		char* initval = (char*) xmlNodeGetContent (m_InitNode);
+		char* endval = (char*) xmlNodeGetContent (m_CurNode);
 		gcpOperation *pOp = NULL;
 		if ((initval && strlen(initval)))
 		{
@@ -276,6 +277,10 @@ bool gcpTextTool::Unselect()
 			pOp->AddNode(m_CurNode);
 			m_CurNode = NULL;
 		}
+		if (initval)
+			xmlFree (initval);
+		if (endval)
+			xmlFree (endval);
 		if (pOp) m_pView->GetDoc()->PushOperation(pOp, m_bUndo);
 		m_bUndo = true;
 	}
@@ -292,6 +297,7 @@ bool gcpTextTool::Unselect()
 		m_pView->GetDoc()->Remove(pObj);
 		m_pView->GetDoc()->AbortOperation();
 	}
+	m_pApp->ActivateMenu ("Image", m_pView->GetDoc()->HasChildren ());
 	return true;
 }
 
@@ -538,8 +544,9 @@ bool gcpTextTool::OnUndo()
 	m_pApp->ActivateMenu("Redo", true);
 	GtkTextBuffer* buf = gnome_canvas_rich_text_ext_get_buffer(m_Active);
 	GtkTextIter cursor;
-	char* tmp = (char*)xmlGetProp(m_CurNode, (xmlChar*)"cursor");
-	int offset = (int)strtoul(tmp, NULL, 10);
+	char* tmp = (char*) xmlGetProp (m_CurNode, (xmlChar*) "cursor");
+	int offset = (int) strtoul (tmp, NULL, 10);
+	xmlFree (tmp);
 	gtk_text_buffer_get_iter_at_offset(buf, &cursor, offset);
 	gtk_text_buffer_place_cursor(buf, &cursor);
 	m_CurNode = node;
@@ -560,8 +567,9 @@ bool gcpTextTool::OnRedo()
 	m_pApp->ActivateMenu("Undo", true);
 	GtkTextBuffer* buf = gnome_canvas_rich_text_ext_get_buffer(m_Active);
 	GtkTextIter cursor;
-	char* tmp = (char*)xmlGetProp(m_CurNode, (xmlChar*)"cursor");
-	int offset = (int)strtoul(tmp, NULL, 10);
+	char* tmp = (char*) xmlGetProp (m_CurNode, (xmlChar*) "cursor");
+	int offset = (int) strtoul (tmp, NULL, 10);
+	xmlFree (tmp);
 	gtk_text_buffer_get_iter_at_offset(buf, &cursor, offset);
 	gtk_text_buffer_place_cursor(buf, &cursor);
 	m_CurNode = node;
